/*
 * Created on Jun 23, 2005
 *
 */
package com.octo.cas.client.websphere;

import java.util.Hashtable;
import java.util.Properties;

import javax.security.auth.Subject;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.security.WebTrustAssociationException;
import com.ibm.websphere.security.WebTrustAssociationFailedException;
import com.ibm.wsspi.security.tai.TAIResult;
import com.ibm.wsspi.security.tai.TrustAssociationInterceptor;
import com.ibm.wsspi.security.token.AttributeNameConstants;

import edu.yale.its.tp.cas.client.ServiceTicketValidator;

/**
 * <a
 * href="http://www-128.ibm.com/developerworks/views/download.jsp?contentid=92125&filename=WASAdvancedAuthentication.zip&method=http&locale=worldwide">
 * Trusted Authentication Interceptor implementation  (WAS 5.1.1 and above)</a> for CAS. <br/>This
 * TAI intercept calls containing the "ticket" parameter (Added by a CAS
 * server after successfull authentication), and check against the CAS server
 * that the ticket is valid for the requested service. 
 * <br/>It also uses this
 * ticket to retrieve the real user name, and build a JAAS Subject using this
 * information. (unless found parameter 'subject=no' in the request). <br/>
 * Finally it add a proxy ticket to the subject public credentials in a
 * {@link CasProxyTicketHolder}(only if 'STORE_PROXY_TICKET=true'). In this case you'll have to deploy a proxy ticket
 * receptor servlet in the target application's web.xml (portal, wcm, or whatever) : see <a
 * href="http://clearinghouse.ja-sig.org:9000/display/CAS/Using+ProxyTicketReceptor">Ticket
 * Proxy Receptor documentation </a> <br/>
 * <br/>See static final properties of {@link CasTAIHelper} 
 * documentation for configuration.
 * 
 * 
 * 
 * @author Marc-Antoine Garrigue
 *  
 */
public class CasTAI511 implements TrustAssociationInterceptor  {

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#initialize(java.util.Properties)
	 */
	public int initialize(Properties props)
			throws WebTrustAssociationFailedException {
		return CasTAIHelper.initialize(props);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#isTargetInterceptor(javax.servlet.http.HttpServletRequest)
	 */
	public boolean isTargetInterceptor(HttpServletRequest request)
			throws WebTrustAssociationException {
		return CasTAIHelper.isTargetInterceptor(request);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#negotiateValidateandEstablishTrust(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public TAIResult negotiateValidateandEstablishTrust(
			HttpServletRequest request, HttpServletResponse response)
			throws WebTrustAssociationFailedException {
		
		/* instantiate a new ServiceTicketValidator */
			ServiceTicketValidator sv = new ServiceTicketValidator();
			String ticket;
			try {
				ticket = CasTAIHelper.getTicket(request);
			} catch (ServiceTicketLostException e) {
				throw new WebTrustAssociationFailedException(e.getMessage());
			}
			String uid;
			try {
				uid = CasTAIHelper.negotiateValidateandEstablishTrust(ticket,request, sv);
			} catch (CasException e1) {
				throw new WebTrustAssociationFailedException(e1.getMessage());
			}
			
			
			if (CasTAIHelper.STORE_PROXY_TICKET) {
				
				if (CasTAIHelper.DEBUG)
					System.out.println("Beginning proxy ticket exchange workflow");
					String pgtiou = sv.getPgtIou();
					if(pgtiou==null){
						System.err
						.println("Unable to retreive the proxy ticket, check the ProxyTicketReceptorConfiguration");
						throw new WebTrustAssociationFailedException(
						"Unable to retrieve the proxy ticket, check the ProxyTicketReceptorConfiguration");
					}
					if (CasTAIHelper.DEBUG)
						System.out.println("Creating custom subject");
					Subject subject = new Subject();
					if (CasTAIHelper.DEBUG)
						System.out.println("adding a CasProxyTicketHolder with pgtiou=" + pgtiou);
					
					
					//use a unique ID for WSCREDENTIAL_CACHE_KEY in order to avoid the default credentials creation.
					Hashtable hashtable= new Hashtable();
					
					hashtable.put(AttributeNameConstants.WSCREDENTIAL_CACHE_KEY,ticket);
					
					subject.getPublicCredentials().add(
							new CasProxyTicketHolder(pgtiou));

					subject.getPublicCredentials().add(hashtable);
					System.out.println("CAS Authentication success, returned principal: '"+uid+"'");
					return TAIResult
							.create(HttpServletResponse.SC_OK, uid, subject);
					
			} else {
				System.out.println("CAS Authentication success, returned principal: '"+uid+"'");
			
				return TAIResult.create(HttpServletResponse.SC_OK, uid);
			}

		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#getVersion()
	 */
	public String getVersion() {
		return "1.0.0";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#getType()
	 */
	public String getType() {
		return "CasTAI511";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#cleanup()
	 */
	public void cleanup() {
	}

}